
!define VERSION "3.0.2"
!define SVCNAME "OpenSSHServer"
!define SVCUSR  "SvcOPENSSHD"
!define PACKAGE "OpenSSHServer"

!define NAME "ICW"
!define UNINSTPROG "uninstall_${NAME}_${PACKAGE}.exe"
!define REGROOT "Software\Itefix\ICW"

!include "${NSISDIR}\Include\WinMessages.nsh"

SetCompressor /SOLID LZMA

!include "MUI.nsh"
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_LANGUAGE "English"

!include "FileFunc.nsh"
!insertmacro GetParameters
!insertmacro GetOptions

Name "${NAME} ${PACKAGE} ${VERSION}"
OutFile "${NAME}_${PACKAGE}_${VERSION}_installer.exe"
InstallDirRegKey HKLM ${REGROOT} "InstallDirectory"
AutoCloseWindow true

VIAddVersionKey  "ProductName" "${NAME}"
VIAddVersionKey  "CompanyName" "ITeF!x Consulting"
VIAddVersionKey  "FileDescription" "${NAME} ${PACKAGE}"
VIAddVersionKey  "FileVersion" "${VERSION}"
VIProductVersion "${VERSION}.0"

Var installtype
Var svcuser
Var svcpassword

Function .onInit

	StrCpy $installtype "fresh"

	Init_Cont_A:
	# Check if icw base is installed 
	ReadRegStr $0 HKLM "${REGROOT}\Base" "version"
	IfErrors 0 Init_Cont_B
	MessageBox MB_OK|MB_ICONSTOP  "${NAME} Base package is required for ${NAME} ${PACKAGE}." /SD IDOK
	Abort
	
	Init_Cont_B:	
	# Check for previous package installations
	ReadRegStr $0 HKLM "${REGROOT}\${PACKAGE}" "version"
	IfErrors Init_Cont_C
	StrCpy $installtype "upgrade"	

	Init_Cont_C:
	# Check if user / password is defined via command line
	${GetParameters} $0
	${GetOptions} $0 "/u="  $svcuser
	${GetOptions} $0 "/p="  $svcpassword
	IfErrors 0 Init_End
	MessageBox MB_OK|MB_ICONSTOP  "No service account and password are specified." /SD IDOK
	Abort

	Init_End:
	
FunctionEnd

Function un.onInit
FunctionEnd

# Install section
Section "${NAME} ${PACKAGE}"

	SetAutoClose true

	StrCmp $installtype "upgrade" 0 Install_A
	IfSilent +2
	Banner::show /NOUNLOAD "Upgrading ${PACKAGE} ..."
	
	Call UpgradePackage
	Goto Install_End
	
	Install_A:
	IfSilent +2
	Banner::show /NOUNLOAD "Installing ${PACKAGE} ..."
	
	Call InstallPackage
	
	Install_End:
	Banner::destroy

	WriteUninstaller "$INSTDIR\${UNINSTPROG}"

SectionEnd

# Uninstall section
Section "Uninstall"

	SetAutoClose true
	
	ReadRegStr $svcuser HKLM "${REGROOT}\${PACKAGE}" "ServiceAccount"

	DetailPrint "Remove registry keys"
	DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${PACKAGE}"
	DeleteRegKey HKLM "${REGROOT}\${PACKAGE}"

	DetailPrint "Remove uninstaller"
	Delete $INSTDIR\${UNINSTPROG}
	
	DetailPrint "Remove user for privilege separation"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c deletesshdaccount'
		
	DetailPrint "Revoke service account rights and permissions"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c removerights --serviceaccount "$svcuser"'
	
	DetailPrint "Stop and remove the service"
	nsExec::ExecToLog '"$INSTDIR\Bin\cygrunsrv" -E ${SVCNAME}'
	nsExec::ExecToLog '"$INSTDIR\Bin\cygrunsrv" -R ${SVCNAME}'
		
	DetailPrint "Remove files"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c setpermissions'

	Call un.DeleteFiles
	
SectionEnd

Function InstallPackage

	Call InstallFiles

	; Add files specific to new installation only or files not to be overwritten
	SetOutPath "$INSTDIR\etc"
	File etc\sshd_config
	File etc\ssh_config
	
	Call SetupService
	
	; Write the version into the registry
	WriteRegStr HKLM "${REGROOT}\${PACKAGE}" "Version" "${VERSION}"
	WriteRegStr HKLM "${REGROOT}\${PACKAGE}" "ServiceAccount" $svcuser

	; Write the uninstall keys for Windows
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${PACKAGE}" "DisplayName" "${NAME} ${PACKAGE} (remove only)"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${PACKAGE}" "UninstallString" '"${UNINSTPROG}"'

	MessageBox MB_OK "${NAME} ${PACKAGE} ${VERSION} is installed as a service. \
		No users are allowed to use ${PACKAGE} by default. \
		Use 'COPSHH Control Panel'  from start menu for user activation." /SD IDOK	

FunctionEnd

Function SetupService
	
	DetailPrint "Initialize passwd and group files"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c initpasswd -i "$INSTDIR"'
	
	DetailPrint "Create directory structure"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c createpaths -i "$INSTDIR"'

	DetailPrint "Generate host keys - RSA"
	nsExec::ExecToLog '"$INSTDIR\Bin\ssh-keygen" -q -t rsa  -f /etc/ssh_host_rsa_key -N $\'$\''
	
	DetailPrint "Generate host keys - DSA"
	nsExec::ExecToLog '"$INSTDIR\Bin\ssh-keygen" -q -t dsa  -f /etc/ssh_host_dsa_key -N $\'$\''
	
	DetailPrint "Generate host keys - ECDSA"
	nsExec::ExecToLog '"$INSTDIR\Bin\ssh-keygen" -q -t ecdsa -f /etc/ssh_host_ecdsa_key -N $\'$\''

	DetailPrint "Generate sshd account"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c createsshdaccount -i "$INSTDIR"'
	
	DetailPrint "Creating service"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" --verbose -c createservice -i "$INSTDIR" --serviceaccount "$svcuser" --servicepassword "$svcpassword" --servicename "${SVCNAME}"'
	
	DetailPrint "Set permissions"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c setpermissions -i "$INSTDIR" --serviceaccount "$svcuser"'
	; workaround cygwin 1.7 upgrade / openssh 5.3
	nsExec::ExecToLog '"$INSTDIR\Bin\chown" -R $svcuser /var'

	DetailPrint "Grant service account necessary rights and permissions"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c grantrights -i "$INSTDIR" --serviceaccount "$svcuser"'

	DetailPrint "Start sshd service"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c startservice -i "$INSTDIR"'
	
FunctionEnd

Function UpgradePackage

	DetailPrint "Stop ${SVCNAME} service"
	nsExec::ExecToLog '"$INSTDIR\Bin\cygrunsrv" -E ${SVCNAME}'
	
	Call InstallFiles
	
	DetailPrint "Set permissions"
	nsExec::ExecToLog '"$INSTDIR\Bin\copsshadm" -c setpermissions -i "$INSTDIR" --serviceaccount "$svcuser"'
	; workaround cygwin 1.7 upgrade / openssh 5.3
	nsExec::ExecToLog '"$INSTDIR\Bin\chown" -R $svcuser /var'

	DetailPrint "Start ${SVCNAME} service"
	nsExec::ExecToLog '"$INSTDIR\Bin\cygrunsrv" -S ${SVCNAME}'

	; Update the version info 
	WriteRegStr HKLM "${REGROOT}\${PACKAGE}" "Version" "${VERSION}"

FunctionEnd

Function InstallFiles

	SetOutPath "$INSTDIR\bin"
	File bin\copsshadm.exe
	File bin\scp.exe
	File bin\sftp-server.exe
	File bin\sftp.exe
	File bin\sftponly
	File bin\ssh-add.exe
	File bin\ssh-agent.exe
	File bin\ssh-keygen.exe
	File bin\ssh-keygen.exe.manifest
	File bin\ssh-keyscan.exe
	File bin\ssh-keysign.exe
	File bin\ssh.exe
	File bin\sshd.exe
		
	SetOutPath "$INSTDIR\etc"
	File etc\moduli
	File etc\profile
	File /r etc\skel
	
	SetOutPath "$INSTDIR\doc"
	File /r doc\openssh
	File /r doc\openssl
	File doc\copsshadm.html
	File doc\moduli.html
	File doc\css1.css
	File doc\scp.html
	File doc\moduli.html
	File doc\sftp-server.html
	File doc\sftp.html
	File doc\ssh-add.html
	File doc\ssh-agent.html
	File doc\ssh-keygen.html
	File doc\ssh-keyscan.html
	File doc\ssh-keysign.html
	File doc\ssh.html
	File doc\sshd.html
	File doc\sshd_config.html
	File doc\ssh_config.html
			
FunctionEnd

Function un.DeleteFiles

	Delete $INSTDIR\bin\copsshadm.exe
	Delete $INSTDIR\bin\scp.exe
	Delete $INSTDIR\bin\sftp-server.exe
	Delete $INSTDIR\bin\sftp.exe
	Delete $INSTDIR\bin\sftponly
	Delete $INSTDIR\bin\ssh-add.exe
	Delete $INSTDIR\bin\ssh-agent.exe
	Delete $INSTDIR\bin\ssh-keygen.exe
	Delete $INSTDIR\bin\ssh-keygen.exe.manifest
	Delete $INSTDIR\bin\ssh-keyscan.exe
	Delete $INSTDIR\bin\ssh-keysign.exe
	Delete $INSTDIR\bin\ssh.exe
	Delete $INSTDIR\bin\sshd.exe
		
	Delete $INSTDIR\etc\passwd
	Delete $INSTDIR\etc\group
	Delete $INSTDIR\etc\moduli
	Delete $INSTDIR\etc\profile
	RMDir /r $INSTDIR\etc\skel
	Delete $INSTDIR\etc\sshd_config
	Delete $INSTDIR\etc\ssh_config
	Delete $INSTDIR\etc\ssh_host_*.*	
	RMDir $INSTDIR\etc ; if empty

	RMDir /r $INSTDIR\doc\openssh
	RMDir /r $INSTDIR\doc\openssl
	
	Delete $INSTDIR\doc\copsshadm.html
	Delete $INSTDIR\doc\moduli.html
	Delete $INSTDIR\doc\css1.css
	Delete $INSTDIR\doc\scp.html
	Delete $INSTDIR\doc\sftp-server.html
	Delete $INSTDIR\doc\sftp.html
	Delete $INSTDIR\doc\ssh-add.html
	Delete $INSTDIR\doc\ssh-agent.html
	Delete $INSTDIR\doc\ssh-keygen.html
	Delete $INSTDIR\doc\ssh-keyscan.html
	Delete $INSTDIR\doc\ssh-keysign.html
	Delete $INSTDIR\doc\ssh.html
	Delete $INSTDIR\doc\sshd.html
	Delete $INSTDIR\doc\sshd_config.html
	Delete $INSTDIR\doc\ssh_config.html
	RMDir $INSTDIR\doc ; if empty
	
	Delete $INSTDIR\var\log\lastlog
	Delete $INSTDIR\var\log\wtmp
	Delete $INSTDIR\var\run\utmp
	RMDir $INSTDIR\var\log ; if empty
	RMDir $INSTDIR\var\run ; if empty
	RMDir $INSTDIR\var\empty ; if empty
	RMDir $INSTDIR\var ; if empty
	RMDir $INSTDIR\home ; if empty
	Delete $INSTDIR\sshd-std*.log
	RMDir $INSTDIR	
	
FunctionEnd
